@@ -9,7 +9,7 @@ import sqlite3
 from collections import defaultdict
 
 import numpy as np
-from lingtrain_aligner import model_dispatcher, vis_helper, preprocessor
+from lingtrain_aligner import model_dispatcher, vis_helper, preprocessor, constants as con
 from scipy import spatial
 
 to_delete = re.compile(
@@ -25,7 +25,7 @@ def clean_lines(lines):
     return [re.sub(to_delete, '', line) for line in lines]
 
 
-def process_batch(lines_from_batch, lines_to_batch, line_ids_from, line_ids_to, batch_number, model_name, window, embed_batch_size, normalize_embeddings, show_progress_bar, save_pic=False, lang_name_from="", lang_name_to="", img_path=""):
+def process_batch(lines_from_batch, lines_to_batch, line_ids_from, line_ids_to, batch_number, model_name, window, embed_batch_size, normalize_embeddings, show_progress_bar, save_pic=False, lang_name_from="", lang_name_to="", img_path="", show_info=False, show_regression=False):
     """Do the actual alignment process logic"""
     # try:
     logging.info(f"Batch {batch_number}. Calculating vectors.")
@@ -45,10 +45,13 @@ def process_batch(lines_from_batch, lines_to_batch, line_ids_from, line_ids_to,
     sim_matrix = get_sim_matrix(vectors1, vectors2, window)
     sim_matrix_best = best_per_row_with_ones(sim_matrix)
 
+    x_min, y_min = min(line_ids_from), min(line_ids_to)
+    x_max, y_max = max(line_ids_from), max(line_ids_to)
+
     # save picture
     if save_pic:
         vis_helper.save_pic(sim_matrix_best, lang_name_to,
-                            lang_name_from, img_path, batch_number, transparent=True)
+                            lang_name_from, img_path, batch_number, (x_min, x_max), (y_min, y_max), transparent=True, show_info=show_info, show_regression=show_regression)
 
     best_sim_ind = sim_matrix_best.argmax(1)
     texts_from = []
@@ -71,7 +74,7 @@ def process_batch(lines_from_batch, lines_to_batch, line_ids_from, line_ids_to,
     #     return [], []
 
 
-def align_texts(splitted_from, splitted_to, model_name, batch_size, window, batch_ids=[], save_pic=False, lang_from="", lang_to="", img_path="", embed_batch_size=10, normalize_embeddings=True, show_progress_bar=False, shift=0):
+def align_texts(splitted_from, splitted_to, model_name, batch_size, window, batch_ids=[], save_pic=False, lang_from="", lang_to="", img_path="", embed_batch_size=10, normalize_embeddings=True, show_progress_bar=False, shift=0, show_info=False, show_regression=False):
     result = []
     task_list = [(lines_from_batch, lines_to_batch, line_ids_from, line_ids_to, batch_id)
                  for lines_from_batch, lines_to_batch,
@@ -80,7 +83,7 @@ def align_texts(splitted_from, splitted_to, model_name, batch_size, window, batc
 
     for lines_from_batch, lines_to_batch, line_ids_from, line_ids_to, batch_id in task_list:
         texts_from, texts_to = process_batch(lines_from_batch, lines_to_batch, line_ids_from,
-                                             line_ids_to, batch_id, model_name, window, embed_batch_size, normalize_embeddings, show_progress_bar, save_pic, lang_from, lang_to, img_path)
+                                             line_ids_to, batch_id, model_name, window, embed_batch_size, normalize_embeddings, show_progress_bar, save_pic, lang_from, lang_to, img_path, show_info=show_info, show_regression=show_regression)
         result.append((batch_id, texts_from, texts_to))
 
     # sort by batch_id (will be useful with parallel processing)
@@ -89,7 +92,7 @@ def align_texts(splitted_from, splitted_to, model_name, batch_size, window, batc
     return result
 
 
-def align_db(db_path, model_name, batch_size, window, batch_ids=[], save_pic=False, lang_from="", lang_to="", img_path="", embed_batch_size=10, normalize_embeddings=True, show_progress_bar=False, shift=0):
+def align_db(db_path, model_name, batch_size, window, batch_ids=[], save_pic=False, lang_from="", lang_to="", img_path="", embed_batch_size=10, normalize_embeddings=True, show_progress_bar=False, shift=0, show_info=False, show_regression=False):
     result = []
     splitted_from = get_splitted_from(db_path)
     splitted_to = get_splitted_to(db_path)
@@ -102,8 +105,8 @@ def align_db(db_path, model_name, batch_size, window, batch_ids=[], save_pic=Fal
     for lines_from_batch, lines_to_batch, line_ids_from, line_ids_to, batch_id in task_list:
         print("batch:", count)
         texts_from, texts_to = process_batch(lines_from_batch, lines_to_batch, line_ids_from,
-                                             line_ids_to, batch_id, model_name, window, embed_batch_size, normalize_embeddings, show_progress_bar, save_pic, lang_from, lang_to, img_path)
-        result.append((batch_id, texts_from, texts_to))
+                                             line_ids_to, batch_id, model_name, window, embed_batch_size, normalize_embeddings, show_progress_bar, save_pic, lang_from, lang_to, img_path, show_info=show_info, show_regression=show_regression)
+        result.append((batch_id, texts_from, texts_to, shift, window))
         count += 1
 
     if not result:
@@ -182,13 +185,13 @@ def get_sim_matrix(vec1, vec2, window):
 
 def save_db(db_path, data):
     with sqlite3.connect(db_path) as db:
-        rewrite_processing_batches(db, data)
+        write_processing_batches(db, data)
         create_doc_index(db, data)
 
 
 def create_doc_index(db, data):
     """Create document index in database"""
-    batch_ids = [batch_id for batch_id, x, y in data]
+    batch_ids = [batch_id for batch_id, _, _, _, _ in data]
 
     max_batch_id = max(batch_ids)
     doc_index = get_doc_index(db)
@@ -225,24 +228,27 @@ def get_doc_index(db):
     return res
 
 
-def rewrite_processing_batches(db, data):
+def write_processing_batches(db, data):
     """Insert or rewrite batched data"""
-    for batch_id, texts_from, texts_to in data:
+    for batch_id, texts_from, texts_to, shift, window in data:
         db.execute("delete from processing_from where batch_id=:batch_id", {
             "batch_id": batch_id})
         db.executemany(
-            f"insert into processing_from(batch_id, text_ids, initial_id, text) values (?,?,?,?)", [(batch_id, a, b, c) for a, b, c in texts_from])
+            "insert into processing_from(batch_id, text_ids, initial_id, text) values (?,?,?,?)", [(batch_id, a, b, c) for a, b, c in texts_from])
         db.execute("delete from processing_to where batch_id=:batch_id", {
             "batch_id": batch_id})
         db.executemany(
-            f"insert into processing_to(batch_id, text_ids, initial_id, text) values (?,?,?,?)", [(batch_id, a, b, c) for a, b, c in texts_to])
-    update_batch_progress(db, batch_id)
+            "insert into processing_to(batch_id, text_ids, initial_id, text) values (?,?,?,?)", [(batch_id, a, b, c) for a, b, c in texts_to])
+        db.execute(
+            "insert or replace into batches (batch_id, insert_ts, shift, window) values (?, datetime('now'), ?, ?)", (batch_id, shift, window))
 
 
-def update_batch_progress(db, batch_id):
+def update_history(db_path, batch_ids, operation, parameters):
     """Update batches table with already processed batches IDs"""
-    db.execute(
-        "insert or ignore into batches (batch_id, insert_ts) values (?, datetime('now'))", (batch_id,))
+    parameters = json.dumps(parameters)
+    with sqlite3.connect(db_path) as db:
+        db.executemany(
+            "insert into history(operation, batch_id, parameters, insert_ts) values (?,?,?, datetime('now'))", [(operation, batch_id, parameters) for batch_id in batch_ids])
 
 
 def init_document_db(db_path):
@@ -261,11 +267,16 @@ def init_document_db(db_path):
         db.execute(
             'create table doc_index(id integer primary key, contents varchar)')
         db.execute(
-            'create table batches(id integer primary key, batch_id integer unique, insert_ts text)')
+            'create table batches(id integer primary key, batch_id integer unique, insert_ts text, shift integer, window integer)')
+        db.execute(
+            'create table history(id integer primary key, operation text, batch_id integer, insert_ts text, parameters text)')
         db.execute(
             'create table meta(id integer primary key, key text, val text, occurence integer, par_id integer)')
         db.execute(
             'create table languages(id integer primary key, key text, val text)')
+        db.execute(
+            'create table version(id integer primary key, version text)')
+        db.execute('insert into version(version) values (?)', (con.DB_VERSION,))
 
 
 def fill_db_from_files(db_path, lang_from, lang_to, splitted_from, splitted_to, proxy_from, proxy_to):
@@ -431,7 +442,9 @@ def update_mark_counter(marks_counter, line, mark):
 def get_mark_value(line, mark):
     res = ''
     ending = f"{preprocessor.PARAGRAPH_MARK}{mark}."
-    if line.endswith(ending):
+    if line.endswith(ending):        
+        if mark == preprocessor.DIVIDER:
+            return '* * *'
         res = line[:len(line)-len(ending)]
     return res
 
