@@ -486,7 +486,7 @@ def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, outpu
             res_html.write("<div class='dt-row'>")
             for lang in lang_ordered:
 
-                res_html.write(f"<div class='par dt-cell'>  ")
+                res_html.write(f"<div class='par dt-cell'><div class='book-par-id'>{real_par_id}</div>")
 
                 for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
                     res_html.write(
@@ -649,7 +649,7 @@ def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, templat
         res_html += "<div class='dt-row'>"
         for lang in lang_ordered:
 
-            res_html += f"<div class='par dt-cell'>"
+            res_html += f"<div class='par dt-cell'><div class='book-par-id'>{real_par_id}</div>"
 
             for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
                 res_html += f"<span class='s s{k%sent_cycle} {template}'>{sent}</span>"
@@ -685,6 +685,15 @@ def write_next_polyheader(writer, next_mark, metas_dict, lang_ordered, add_strin
     else:
         writer.write("<div class='dt-row header'>")
     for lang in lang_ordered:
+        #divider is a special case
+        if next_mark == preprocessor.DIVIDER:
+            el = f"<div class='dt-cell divider'><img class='divider-img' src='{DIVIDER_URL}'/></div>"
+            if next_mark in metas[lang] and metas[lang][next_mark]:
+                metas[lang][next_mark].pop(0)
+            if add_string: writer += el
+            else: writer.write(el)
+            continue
+
         meta = metas[lang][next_mark]
         if meta:
             val = meta.pop(0)
@@ -696,8 +705,6 @@ def write_next_polyheader(writer, next_mark, metas_dict, lang_ordered, add_strin
             el = f"<div class='par dt-cell'><div class='lt-quote lt-quote-name'>{val[0]}</div></div>"
         elif next_mark == preprocessor.IMAGE:
             el = f"<div class='par dt-cell text-center'><img class='lt-image' src='{val[0]}'/></div>"
-        elif next_mark == preprocessor.DIVIDER:
-            el = f"<div class='dt-cell divider'><img class='divider-img' src='{DIVIDER_URL}'/></div>"
         else:
             el = f"<div class='par dt-cell'><{HEADER_HTML_MAPPING[next_mark]}>{val[0]}</{HEADER_HTML_MAPPING[next_mark]}></div>"
 
@@ -838,6 +845,7 @@ h3 {
 }
 .dt-row {
     display: table-row;
+    position: relative;
 }
 
 # .dt-row:nth-child(even):not(.header) {
@@ -849,6 +857,19 @@ h3 {
     border-bottom: 1px solid #f0f0f0;
 }
 
+.dt-row:hover > .dt-cell > .book-par-id {
+    opacity: 1;
+}
+
+.book-par-id {
+    position: absolute;
+    left: -20px;
+    top: 0;
+    font-size: 12px;
+    opacity: 0;
+    font-weight: 600;
+}
+
 .dt-cell {
     display: table-cell;
     width: %COL_WIDTH% !important;
